@@ -1,7 +1,7 @@
 /*
  *  CUnit - A Unit testing framework library for C.
- *  Copyright (C) 2001  Anil Kumar
- *  Copyright (C) 2004,2005,2006  Anil Kumar, Jerry St.Clair
+ *  Copyright (C) 2001       Anil Kumar
+ *  Copyright (C) 2004-2006  Anil Kumar, Jerry St.Clair
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -35,6 +35,8 @@
  *                none has been supplied. (JDS)
  *
  *  30-Apr-2005   Added notification of failed suite cleanup function. (JDS)
+ *
+ *  02-May-2006   Added internationalization hooks.  (JDS)
  */
 
 /** @file
@@ -54,18 +56,28 @@
 
 #include "CUnit.h"
 #include "TestDB.h"
+#include "MyMem.h"
 #include "Util.h"
 #include "TestRun.h"
 #include "Automated.h"
+#include "CUnit_intl.h"
 
+#define MAX_FILENAME_LENGTH		1025
+
+/*=================================================================
+ *  Global / Static data definitions
+ *=================================================================*/
 static CU_pSuite f_pRunningSuite = NULL;                    /**< The running test suite. */
 static char      f_szDefaultFileRoot[] = "CUnitAutomated";  /**< Default filename root for automated output files. */
-static char      f_szTestListFileName[FILENAME_MAX] = "";   /**< Current output file name for the test listing file. */
-static char      f_szTestResultFileName[FILENAME_MAX] = ""; /**< Current output file name for the test results file. */
+static char      f_szTestListFileName[MAX_FILENAME_LENGTH] = "";   /**< Current output file name for the test listing file. */
+static char      f_szTestResultFileName[MAX_FILENAME_LENGTH] = ""; /**< Current output file name for the test results file. */
 static FILE*     f_pTestResultFile = NULL;                  /**< FILE pointer the test results file. */
 
 static CU_BOOL f_bWriting_CUNIT_RUN_SUITE = CU_FALSE;       /**< Flag for keeping track of when a closing xml tag is required. */
 
+/*=================================================================
+ *  Static function forward declarations
+ *=================================================================*/
 static CU_ErrorCode automated_list_all_tests(CU_pTestRegistry pRegistry, const char* szFilename);
 
 static CU_ErrorCode initialize_result_file(const char* szFilename);
@@ -79,16 +91,9 @@ static void automated_all_tests_complete_message_handler(const CU_pFailureRecord
 static void automated_suite_init_failure_message_handler(const CU_pSuite pSuite);
 static void automated_suite_cleanup_failure_message_handler(const CU_pSuite pSuite);
 
-/*------------------------------------------------------------------------*/
-/** Run CUnit tests using the automated interface.
- *  This function sets appropriate callback functions,
- *  initializes the test output files, and calls the
- *  appropriate functions to list the tests and run them.
- *  If an output file name root has not been specified using
- *  CU_set_output_filename(), a generic root will be applied.
- *  It is an error to call this function before the CUnit
- *  test registry has been initialized (check by assertion).
- */
+/*=================================================================
+ *  Public Interface functions
+ *=================================================================*/
 void CU_automated_run_tests(void)
 {
   assert(NULL != CU_get_registry());
@@ -103,7 +108,7 @@ void CU_automated_run_tests(void)
   }
 
   if (CUE_SUCCESS != initialize_result_file(f_szTestResultFileName)) {
-    fprintf(stderr, "\nERROR - Failed to create/initialize the result file.");
+    fprintf(stderr, "\n%s", _("ERROR - Failed to create/initialize the result file."));
   }
   else {
     /* set up the message handlers for writing xml output */
@@ -111,25 +116,19 @@ void CU_automated_run_tests(void)
     CU_set_test_complete_handler(automated_test_complete_message_handler);
     CU_set_all_test_complete_handler(automated_all_tests_complete_message_handler);
     CU_set_suite_init_failure_handler(automated_suite_init_failure_message_handler);
-    CU_set_suite_cleanup_failure_handler(automated_suite_cleanup_failure_message_handler);
+    CU_set_suite_cleanup_failure_handler(automated_suite_cleanup_failure_message_handler);   
 
     f_bWriting_CUNIT_RUN_SUITE = CU_FALSE;
 
     automated_run_all_tests(NULL);
-
+    
     if (CUE_SUCCESS != uninitialize_result_file()) {
-      fprintf(stderr, "\nERROR - Failed to close/uninitialize the result files.");
+      fprintf(stderr, "\n%s", _("ERROR - Failed to close/uninitialize the result files."));
     }
   }
 }
 
 /*------------------------------------------------------------------------*/
-/** Set the root file name for automated test output files.
- *  The strings "-Listing.xml" and "-Results.xml" are appended to
- *  the specified root to generate the filenames.  If szFilenameRoot
- *  is empty, the default root ("CUnitAutomated") is used.
- *  @param szFilenameRoot String containing root to use for file names.
- */
 void CU_set_output_filename(const char* szFilenameRoot)
 {
   const char* szListEnding = "-Listing.xml";
@@ -137,35 +136,28 @@ void CU_set_output_filename(const char* szFilenameRoot)
 
   /* Construct the name for the listing file */
   if (NULL != szFilenameRoot) {
-    strncpy(f_szTestListFileName, szFilenameRoot, FILENAME_MAX - strlen(szListEnding) - 1);
+    strncpy(f_szTestListFileName, szFilenameRoot, MAX_FILENAME_LENGTH - strlen(szListEnding) - 1);
   }
   else {
-    strncpy(f_szTestListFileName, f_szDefaultFileRoot, FILENAME_MAX - strlen(szListEnding) - 1);
+    strncpy(f_szTestListFileName, f_szDefaultFileRoot, MAX_FILENAME_LENGTH - strlen(szListEnding) - 1);
   }
 
-  f_szTestListFileName[FILENAME_MAX - strlen(szListEnding) - 1] = '\0';
+  f_szTestListFileName[MAX_FILENAME_LENGTH - strlen(szListEnding) - 1] = '\0';
   strcat(f_szTestListFileName, szListEnding);
 
   /* Construct the name for the result file */
   if (NULL != szFilenameRoot) {
-    strncpy(f_szTestResultFileName, szFilenameRoot, FILENAME_MAX - strlen(szResultEnding) - 1);
+    strncpy(f_szTestResultFileName, szFilenameRoot, MAX_FILENAME_LENGTH - strlen(szResultEnding) - 1);
   }
   else {
-    strncpy(f_szTestResultFileName, f_szDefaultFileRoot, FILENAME_MAX - strlen(szResultEnding) - 1);
+    strncpy(f_szTestResultFileName, f_szDefaultFileRoot, MAX_FILENAME_LENGTH - strlen(szResultEnding) - 1);
   }
 
-  f_szTestResultFileName[FILENAME_MAX - strlen(szResultEnding) - 1] = '\0';
+  f_szTestResultFileName[MAX_FILENAME_LENGTH - strlen(szResultEnding) - 1] = '\0';
   strcat(f_szTestResultFileName, szResultEnding);
 }
 
 /*------------------------------------------------------------------------*/
-/** Generate an xml file containing a list of all tests in all
- *  suites in the active registry.
- *  The output file will be named according to the most recent
- *  call to CU_set_output_filename(), or a default if not
- *  previously set.
- *  @return An error code indicating the error status.
- */
 CU_ErrorCode CU_list_tests_to_file()
 {
   /* if a filename root hasn't been set, use the default one */
@@ -176,8 +168,10 @@ CU_ErrorCode CU_list_tests_to_file()
   return automated_list_all_tests(CU_get_registry(), f_szTestListFileName);
 }
 
-/*------------------------------------------------------------------------*/
-/** Run the registered tests using the automated interface.
+/*=================================================================
+ *  Static function implementation
+ *=================================================================*/
+/** Runs the registered tests using the automated interface.
  *  If non-NULL. the specified registry is set as the active 
  *  registry for running the tests.  If NULL, then the default
  *  CUnit test registry is used.  The actual test running is 
@@ -203,7 +197,7 @@ static void automated_run_all_tests(CU_pTestRegistry pRegistry)
 }
 
 /*------------------------------------------------------------------------*/
-/** Initialize the test results file generated by the automated interface.
+/** Initializes the test results file generated by the automated interface.
  *  A file stream is opened and header information is written.
  */
 static CU_ErrorCode initialize_result_file(const char* szFilename)
@@ -244,6 +238,7 @@ static void automated_test_start_message_handler(const CU_pTest pTest, const CU_
 
   assert(NULL != pTest);
   assert(NULL != pSuite);
+  assert(NULL != pSuite->pName);
   assert(NULL != f_pTestResultFile);
 
   /* write suite close/open tags if this is the 1st test for this szSuite */
@@ -258,7 +253,7 @@ static void automated_test_start_message_handler(const CU_pTest pTest, const CU_
             "    <CUNIT_RUN_SUITE> \n"
             "      <CUNIT_RUN_SUITE_SUCCESS> \n"
             "        <SUITE_NAME> %s </SUITE_NAME> \n",
-            (NULL != pSuite->pName) ? pSuite->pName : "");
+            pSuite->pName);
 
     f_bWriting_CUNIT_RUN_SUITE = CU_TRUE;
     f_pRunningSuite = pSuite;
@@ -275,26 +270,44 @@ static void automated_test_complete_message_handler(const CU_pTest pTest,
                                                     const CU_pSuite pSuite,
                                                     const CU_pFailureRecord pFailure)
 {
+  char *szTemp = NULL;
+  size_t szTemp_len = 0;
+  size_t cur_len = 0;
   CU_pFailureRecord pTempFailure = pFailure;
 
   CU_UNREFERENCED_PARAMETER(pSuite);  /* pSuite is not used except in assertion */
 
   assert(NULL != pTest);
+  assert(NULL != pTest->pName);
   assert(NULL != pSuite);
+  assert(NULL != pSuite->pName);
   assert(NULL != f_pTestResultFile);
 
   if (NULL != pTempFailure) {
 
-    /* worst cast is a string of special chars */
-    char szTemp[CUNIT_MAX_ENTITY_LEN * CUNIT_MAX_STRING_LENGTH];
-
     while (NULL != pTempFailure) {
 
       assert((NULL != pTempFailure->pSuite) && (pTempFailure->pSuite == pSuite));
       assert((NULL != pTempFailure->pTest) && (pTempFailure->pTest == pTest));
 
+      /* expand temporary char buffer if need more room */
       if (NULL != pTempFailure->strCondition) {
-        CU_translate_special_characters(pTempFailure->strCondition, szTemp, sizeof(szTemp));
+        cur_len = CU_translated_strlen(pTempFailure->strCondition) + 1;
+      }
+      else {
+        cur_len = 1;
+      }
+      if (cur_len > szTemp_len) {
+        szTemp_len = cur_len;
+        if (NULL != szTemp) {
+          CU_FREE(szTemp);
+        }
+        szTemp = (char *)CU_MALLOC(szTemp_len);
+      }
+
+      /* convert xml entities in strCondition (if present) */
+      if (NULL != pTempFailure->strCondition) {
+        CU_translate_special_characters(pTempFailure->strCondition, szTemp, szTemp_len);
       }
       else {
         szTemp[0] = '\0';
@@ -309,7 +322,7 @@ static void automated_test_complete_message_handler(const CU_pTest pTest,
               "            <CONDITION> %s </CONDITION> \n"
               "          </CUNIT_RUN_TEST_FAILURE> \n"
               "        </CUNIT_RUN_TEST_RECORD> \n",
-              (NULL != pTest->pName) ? pTest->pName : "",
+              pTest->pName,
               (NULL != pTempFailure->strFileName) ? pTempFailure->strFileName : "",
               pTempFailure->uiLineNumber,
               szTemp);
@@ -323,7 +336,11 @@ static void automated_test_complete_message_handler(const CU_pTest pTest,
             "            <TEST_NAME> %s </TEST_NAME> \n"
             "          </CUNIT_RUN_TEST_SUCCESS> \n"
             "        </CUNIT_RUN_TEST_RECORD> \n",
-            (NULL != pTest->pName) ? pTest->pName : "");
+            pTest->pName);
+  }
+
+  if (NULL != szTemp) {
+    CU_FREE(szTemp);
   }
 }
 
@@ -354,44 +371,53 @@ static void automated_all_tests_complete_message_handler(const CU_pFailureRecord
 
   fprintf(f_pTestResultFile,
           "    <CUNIT_RUN_SUMMARY_RECORD> \n"
-          "      <TYPE> Suites </TYPE> \n"
+          "      <TYPE> %s </TYPE> \n"
           "      <TOTAL> %u </TOTAL> \n"
           "      <RUN> %u </RUN> \n"
           "      <SUCCEEDED> - NA - </SUCCEEDED> \n"
           "      <FAILED> %u </FAILED> \n"
+          "      <INACTIVE> %u </INACTIVE> \n"
           "    </CUNIT_RUN_SUMMARY_RECORD> \n",
+          _("Suites"),
           pRegistry->uiNumberOfSuites,
           pRunSummary->nSuitesRun,
-          pRunSummary->nSuitesFailed
+          pRunSummary->nSuitesFailed,
+          pRunSummary->nSuitesInactive
           );
 
   fprintf(f_pTestResultFile,
           "    <CUNIT_RUN_SUMMARY_RECORD> \n"
-          "      <TYPE> Test Cases </TYPE> \n"
+          "      <TYPE> %s </TYPE> \n"
           "      <TOTAL> %u </TOTAL> \n"
           "      <RUN> %u </RUN> \n"
           "      <SUCCEEDED> %u </SUCCEEDED> \n"
           "      <FAILED> %u </FAILED> \n"
+          "      <INACTIVE> %u </INACTIVE> \n"
           "    </CUNIT_RUN_SUMMARY_RECORD> \n",
+          _("Test Cases"),
           pRegistry->uiNumberOfTests,
           pRunSummary->nTestsRun,
           pRunSummary->nTestsRun - pRunSummary->nTestsFailed,
-          pRunSummary->nTestsFailed
+          pRunSummary->nTestsFailed,
+          pRunSummary->nTestsInactive
           );
 
   fprintf(f_pTestResultFile,
           "    <CUNIT_RUN_SUMMARY_RECORD> \n"
-          "      <TYPE> Assertions </TYPE> \n"
+          "      <TYPE> %s </TYPE> \n"
           "      <TOTAL> %u </TOTAL> \n"
           "      <RUN> %u </RUN> \n"
           "      <SUCCEEDED> %u </SUCCEEDED> \n"
           "      <FAILED> %u </FAILED> \n"
+          "      <INACTIVE> %s </INACTIVE> \n"
           "    </CUNIT_RUN_SUMMARY_RECORD> \n"
           "  </CUNIT_RUN_SUMMARY> \n",
+          _("Assertions"),
           pRunSummary->nAsserts,
           pRunSummary->nAsserts,
           pRunSummary->nAsserts - pRunSummary->nAssertsFailed,
-          pRunSummary->nAssertsFailed
+          pRunSummary->nAssertsFailed,
+          _("n/a")
           );
 }
 
@@ -402,6 +428,7 @@ static void automated_all_tests_complete_message_handler(const CU_pFailureRecord
 static void automated_suite_init_failure_message_handler(const CU_pSuite pSuite)
 {
   assert(NULL != pSuite);
+  assert(NULL != pSuite->pName);
   assert(NULL != f_pTestResultFile);
 
   if (CU_TRUE == f_bWriting_CUNIT_RUN_SUITE) {
@@ -418,8 +445,8 @@ static void automated_suite_init_failure_message_handler(const CU_pSuite pSuite)
           "        <FAILURE_REASON> %s </FAILURE_REASON> \n"
           "      </CUNIT_RUN_SUITE_FAILURE> \n"
           "    </CUNIT_RUN_SUITE>  \n",
-          (NULL != pSuite->pName) ? pSuite->pName : "", 
-          "Suite Initialization Failed");
+           pSuite->pName, 
+          _("Suite Initialization Failed"));
 }
 
 /*------------------------------------------------------------------------*/
@@ -429,6 +456,7 @@ static void automated_suite_init_failure_message_handler(const CU_pSuite pSuite)
 static void automated_suite_cleanup_failure_message_handler(const CU_pSuite pSuite)
 {
   assert(NULL != pSuite);
+  assert(NULL != pSuite->pName);
   assert(NULL != f_pTestResultFile);
 
   if (CU_TRUE == f_bWriting_CUNIT_RUN_SUITE) {
@@ -445,12 +473,12 @@ static void automated_suite_cleanup_failure_message_handler(const CU_pSuite pSui
           "        <FAILURE_REASON> %s </FAILURE_REASON> \n"
           "      </CUNIT_RUN_SUITE_FAILURE> \n"
           "    </CUNIT_RUN_SUITE>  \n",
-          (NULL != pSuite->pName) ? pSuite->pName : "", 
-          "Suite Cleanup Failed");
+          pSuite->pName, 
+          _("Suite Cleanup Failed"));
 }
 
 /*------------------------------------------------------------------------*/
-/** Finalize and close the results output file generated
+/** Finalizes and closes the results output file generated
  *  by the automated interface.
  */
 static CU_ErrorCode uninitialize_result_file(void)
@@ -465,8 +493,9 @@ static CU_ErrorCode uninitialize_result_file(void)
   time(&tTime);
   szTime = ctime(&tTime);
   fprintf(f_pTestResultFile,
-          "  <CUNIT_FOOTER> File Generated By CUnit v" CU_VERSION " at %s </CUNIT_FOOTER> \n"
+          "  <CUNIT_FOOTER> %s" CU_VERSION " - %s </CUNIT_FOOTER> \n"
           "</CUNIT_TEST_RUN_REPORT>",
+          _("File Generated By CUnit v"),
           (NULL != szTime) ? szTime : "");
 
   if (0 != fclose(f_pTestResultFile)) {
@@ -477,7 +506,7 @@ static CU_ErrorCode uninitialize_result_file(void)
 }
 
 /*------------------------------------------------------------------------*/
-/** Generate an xml listing of all tests in all suites for the
+/** Generates an xml listing of all tests in all suites for the
  *  specified test registry.  The output is directed to a file
  *  having the specified name.
  *  @param pRegistry   Test registry for which to generate list (non-NULL).
@@ -517,17 +546,19 @@ static CU_ErrorCode automated_list_all_tests(CU_pTestRegistry pRegistry, const c
 
     fprintf(pTestListFile,
             "    <CUNIT_LIST_TOTAL_SUMMARY_RECORD> \n"
-            "      <CUNIT_LIST_TOTAL_SUMMARY_RECORD_TEXT> Total Number of Suites </CUNIT_LIST_TOTAL_SUMMARY_RECORD_TEXT> \n"
+            "      <CUNIT_LIST_TOTAL_SUMMARY_RECORD_TEXT> %s </CUNIT_LIST_TOTAL_SUMMARY_RECORD_TEXT> \n"
             "      <CUNIT_LIST_TOTAL_SUMMARY_RECORD_VALUE> %u </CUNIT_LIST_TOTAL_SUMMARY_RECORD_VALUE> \n"
             "    </CUNIT_LIST_TOTAL_SUMMARY_RECORD> \n",
+            _("Total Number of Suites"),
             pRegistry->uiNumberOfSuites);
 
     fprintf(pTestListFile,
             "    <CUNIT_LIST_TOTAL_SUMMARY_RECORD> \n"
-            "      <CUNIT_LIST_TOTAL_SUMMARY_RECORD_TEXT> Total Number of Test Cases </CUNIT_LIST_TOTAL_SUMMARY_RECORD_TEXT> \n"
+            "      <CUNIT_LIST_TOTAL_SUMMARY_RECORD_TEXT> %s </CUNIT_LIST_TOTAL_SUMMARY_RECORD_TEXT> \n"
             "      <CUNIT_LIST_TOTAL_SUMMARY_RECORD_VALUE> %u </CUNIT_LIST_TOTAL_SUMMARY_RECORD_VALUE> \n"
             "    </CUNIT_LIST_TOTAL_SUMMARY_RECORD> \n"
             "  </CUNIT_LIST_TOTAL_SUMMARY> \n",
+            _("Total Number of Test Cases"),
             pRegistry->uiNumberOfTests);
 
     fprintf(pTestListFile,
@@ -535,6 +566,7 @@ static CU_ErrorCode automated_list_all_tests(CU_pTestRegistry pRegistry, const c
 
     pSuite = pRegistry->pSuite;
     while (NULL != pSuite) {
+      assert(NULL != pSuite->pName);
       pTest = pSuite->pTest;
 
       fprintf(pTestListFile,
@@ -542,20 +574,27 @@ static CU_ErrorCode automated_list_all_tests(CU_pTestRegistry pRegistry, const c
               "      <CUNIT_ALL_TEST_LISTING_SUITE_DEFINITION> \n"
               "        <SUITE_NAME> %s </SUITE_NAME> \n"
               "        <INITIALIZE_VALUE> %s </INITIALIZE_VALUE> \n"
-              "        <CLEANUP_VALUE>  %s </CLEANUP_VALUE> \n"
+              "        <CLEANUP_VALUE> %s </CLEANUP_VALUE> \n"
+              "        <ACTIVE_VALUE> %s </ACTIVE_VALUE> \n"
               "        <TEST_COUNT_VALUE> %u </TEST_COUNT_VALUE> \n"
               "      </CUNIT_ALL_TEST_LISTING_SUITE_DEFINITION> \n",
-              (NULL != pSuite->pName) ? pSuite->pName : "",
-              (NULL != pSuite->pInitializeFunc) ? "Yes" : "No",
-              (NULL != pSuite->pCleanupFunc) ? "Yes" : "No",
+               pSuite->pName,
+              (NULL != pSuite->pInitializeFunc) ? _("Yes") : _("No"),
+              (NULL != pSuite->pCleanupFunc) ? _("Yes") : _("No"),
+              (CU_FALSE != pSuite->fActive) ? _("Yes") : _("No"),
               pSuite->uiNumberOfTests);
 
       fprintf(pTestListFile,
               "      <CUNIT_ALL_TEST_LISTING_SUITE_TESTS> \n");
       while (NULL != pTest) {
+        assert(NULL != pTest->pName);
         fprintf(pTestListFile,
-                "        <TEST_CASE_NAME> %s </TEST_CASE_NAME> \n", 
-                (NULL != pTest->pName) ? pTest->pName : "");
+                "        <TEST_CASE_DEFINITION> \n"
+                "          <TEST_CASE_NAME> %s </TEST_CASE_NAME> \n"
+                "          <TEST_ACTIVE_VALUE> %s </TEST_ACTIVE_VALUE> \n"
+                "        </TEST_CASE_DEFINITION> \n", 
+                pTest->pName,
+                (CU_FALSE != pSuite->fActive) ? _("Yes") : _("No"));
         pTest = pTest->pNext;
       }
 
@@ -571,8 +610,9 @@ static CU_ErrorCode automated_list_all_tests(CU_pTestRegistry pRegistry, const c
     time(&tTime);
     szTime = ctime(&tTime);
     fprintf(pTestListFile,
-            "  <CUNIT_FOOTER> File Generated By CUnit v" CU_VERSION " at %s </CUNIT_FOOTER> \n"
+            "  <CUNIT_FOOTER> %s" CU_VERSION " - %s </CUNIT_FOOTER> \n"
             "</CUNIT_TEST_LIST_REPORT>",
+            _("File Generated By CUnit v"),
             (NULL != szTime) ? szTime : "");
 
     if (0 != fclose(pTestListFile)) {
